@@ -4,7 +4,7 @@ use std::{
     ops::{Range, RangeInclusive},
 };
 
-use bytes::{Buf, BufMut, Bytes};
+use bytes::{Buf, BufMut, Bytes, BytesMut};
 use tinyvec::TinyVec;
 
 use crate::{
@@ -40,7 +40,7 @@ impl Type {
 
 impl coding::Codec for Type {
     fn decode<B: Buf>(buf: &mut B) -> coding::Result<Self> {
-        Ok(Type(buf.get_var()?))
+        Ok(Self(buf.get_var()?))
     }
     fn encode<B: BufMut>(&self, buf: &mut B) {
         buf.write_var(self.0);
@@ -136,7 +136,7 @@ const STREAM_TYS: RangeInclusive<u64> = RangeInclusive::new(0x08, 0x0f);
 const DATAGRAM_TYS: RangeInclusive<u64> = RangeInclusive::new(0x30, 0x31);
 
 #[derive(Debug)]
-pub enum Frame {
+pub(crate) enum Frame {
     Padding,
     Ping,
     Ack(Ack),
@@ -162,7 +162,7 @@ pub enum Frame {
 }
 
 impl Frame {
-    pub fn ty(&self) -> Type {
+    pub(crate) fn ty(&self) -> Type {
         use self::Frame::*;
         match *self {
             Padding => Type::PADDING,
@@ -202,8 +202,8 @@ impl Frame {
         }
     }
 
-    pub fn is_ack_eliciting(&self) -> bool {
-        !matches!(*self, Frame::Ack(_) | Frame::Padding | Frame::Close(_))
+    pub(crate) fn is_ack_eliciting(&self) -> bool {
+        !matches!(*self, Self::Ack(_) | Self::Padding | Self::Close(_))
     }
 }
 
@@ -216,25 +216,25 @@ pub enum Close {
 impl Close {
     pub(crate) fn encode<W: BufMut>(&self, out: &mut W, max_len: usize) {
         match *self {
-            Close::Connection(ref x) => x.encode(out, max_len),
-            Close::Application(ref x) => x.encode(out, max_len),
+            Self::Connection(ref x) => x.encode(out, max_len),
+            Self::Application(ref x) => x.encode(out, max_len),
         }
     }
 }
 
 impl From<TransportError> for Close {
     fn from(x: TransportError) -> Self {
-        Close::Connection(x.into())
+        Self::Connection(x.into())
     }
 }
 impl From<ConnectionClose> for Close {
     fn from(x: ConnectionClose) -> Self {
-        Close::Connection(x)
+        Self::Connection(x)
     }
 }
 impl From<ApplicationClose> for Close {
     fn from(x: ApplicationClose) -> Self {
-        Close::Application(x)
+        Self::Application(x)
     }
 }
 
@@ -262,7 +262,7 @@ impl fmt::Display for ConnectionClose {
 
 impl From<TransportError> for ConnectionClose {
     fn from(x: TransportError) -> Self {
-        ConnectionClose {
+        Self {
             error_code: x.code,
             frame_type: x.frame,
             reason: x.reason.into(),
@@ -321,8 +321,7 @@ impl ApplicationClose {
     pub(crate) fn encode<W: BufMut>(&self, out: &mut W, max_len: usize) {
         out.write(Type::APPLICATION_CLOSE); // 1 byte
         out.write(self.error_code); // <= 8 bytes
-        let max_len =
-            max_len as usize - 3 - VarInt::from_u64(self.reason.len() as u64).unwrap().size();
+        let max_len = max_len - 3 - VarInt::from_u64(self.reason.len() as u64).unwrap().size();
         let actual_len = self.reason.len().min(max_len);
         out.write_var(actual_len as u64); // <= 8 bytes
         out.put_slice(&self.reason[0..actual_len]); // whatever's left
@@ -345,7 +344,7 @@ impl fmt::Debug for Ack {
             if !first {
                 ranges.push(',');
             }
-            write!(ranges, "{:?}", range).unwrap();
+            write!(ranges, "{range:?}").unwrap();
             first = false;
         }
         ranges.push(']');
@@ -443,11 +442,11 @@ impl EcnCounts {
 }
 
 #[derive(Debug, Clone)]
-pub struct Stream {
-    pub id: StreamId,
-    pub offset: u64,
-    pub fin: bool,
-    pub data: Bytes,
+pub(crate) struct Stream {
+    pub(crate) id: StreamId,
+    pub(crate) offset: u64,
+    pub(crate) fin: bool,
+    pub(crate) data: Bytes,
 }
 
 impl FrameStruct for Stream {
@@ -456,16 +455,16 @@ impl FrameStruct for Stream {
 
 /// Metadata from a stream frame
 #[derive(Debug, Clone)]
-pub struct StreamMeta {
-    pub id: StreamId,
-    pub offsets: Range<u64>,
-    pub fin: bool,
+pub(crate) struct StreamMeta {
+    pub(crate) id: StreamId,
+    pub(crate) offsets: Range<u64>,
+    pub(crate) fin: bool,
 }
 
 // This manual implementation exists because `Default` is not implemented for `StreamId`
 impl Default for StreamMeta {
     fn default() -> Self {
-        StreamMeta {
+        Self {
             id: StreamId(0),
             offsets: 0..0,
             fin: false,
@@ -474,7 +473,7 @@ impl Default for StreamMeta {
 }
 
 impl StreamMeta {
-    pub fn encode<W: BufMut>(&self, length: bool, out: &mut W) {
+    pub(crate) fn encode<W: BufMut>(&self, length: bool, out: &mut W) {
         let mut ty = *STREAM_TYS.start();
         if self.offsets.start != 0 {
             ty |= 0x04;
@@ -497,18 +496,18 @@ impl StreamMeta {
 }
 
 /// A vector of [`StreamMeta`] with optimization for the single element case
-pub type StreamMetaVec = TinyVec<[StreamMeta; 1]>;
+pub(crate) type StreamMetaVec = TinyVec<[StreamMeta; 1]>;
 
 #[derive(Debug, Clone)]
-pub struct Crypto {
-    pub offset: u64,
-    pub data: Bytes,
+pub(crate) struct Crypto {
+    pub(crate) offset: u64,
+    pub(crate) data: Bytes,
 }
 
 impl Crypto {
-    pub const SIZE_BOUND: usize = 17;
+    pub(crate) const SIZE_BOUND: usize = 17;
 
-    pub fn encode<W: BufMut>(&self, out: &mut W) {
+    pub(crate) fn encode<W: BufMut>(&self, out: &mut W) {
         out.write(Type::CRYPTO);
         out.write_var(self.offset);
         out.write_var(self.data.len() as u64);
@@ -516,7 +515,7 @@ impl Crypto {
     }
 }
 
-pub struct Iter {
+pub(crate) struct Iter {
     // TODO: ditch io::Cursor after bytes 0.5
     bytes: io::Cursor<Bytes>,
     last_ty: Option<Type>,
@@ -541,13 +540,13 @@ impl IterErr {
 
 impl From<UnexpectedEnd> for IterErr {
     fn from(_: UnexpectedEnd) -> Self {
-        IterErr::UnexpectedEnd
+        Self::UnexpectedEnd
     }
 }
 
 impl Iter {
-    pub fn new(payload: Bytes) -> Self {
-        Iter {
+    pub(crate) fn new(payload: Bytes) -> Self {
+        Self {
             bytes: io::Cursor::new(payload),
             last_ty: None,
         }
@@ -780,12 +779,13 @@ impl<'a> Iterator for AckIter<'a> {
     }
 }
 
+#[allow(unreachable_pub)] // fuzzing only
 #[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
 #[derive(Debug, Copy, Clone)]
 pub struct ResetStream {
-    pub id: StreamId,
-    pub error_code: VarInt,
-    pub final_offset: VarInt,
+    pub(crate) id: StreamId,
+    pub(crate) error_code: VarInt,
+    pub(crate) final_offset: VarInt,
 }
 
 impl FrameStruct for ResetStream {
@@ -793,7 +793,7 @@ impl FrameStruct for ResetStream {
 }
 
 impl ResetStream {
-    pub fn encode<W: BufMut>(&self, out: &mut W) {
+    pub(crate) fn encode<W: BufMut>(&self, out: &mut W) {
         out.write(Type::RESET_STREAM); // 1 byte
         out.write(self.id); // <= 8 bytes
         out.write(self.error_code); // <= 8 bytes
@@ -802,9 +802,9 @@ impl ResetStream {
 }
 
 #[derive(Debug, Copy, Clone)]
-pub struct StopSending {
-    pub id: StreamId,
-    pub error_code: VarInt,
+pub(crate) struct StopSending {
+    pub(crate) id: StreamId,
+    pub(crate) error_code: VarInt,
 }
 
 impl FrameStruct for StopSending {
@@ -812,7 +812,7 @@ impl FrameStruct for StopSending {
 }
 
 impl StopSending {
-    pub fn encode<W: BufMut>(&self, out: &mut W) {
+    pub(crate) fn encode<W: BufMut>(&self, out: &mut W) {
         out.write(Type::STOP_SENDING); // 1 byte
         out.write(self.id); // <= 8 bytes
         out.write(self.error_code) // <= 8 bytes
@@ -820,15 +820,15 @@ impl StopSending {
 }
 
 #[derive(Debug, Copy, Clone)]
-pub struct NewConnectionId {
-    pub sequence: u64,
-    pub retire_prior_to: u64,
-    pub id: ConnectionId,
-    pub reset_token: ResetToken,
+pub(crate) struct NewConnectionId {
+    pub(crate) sequence: u64,
+    pub(crate) retire_prior_to: u64,
+    pub(crate) id: ConnectionId,
+    pub(crate) reset_token: ResetToken,
 }
 
 impl NewConnectionId {
-    pub fn encode<W: BufMut>(&self, out: &mut W) {
+    pub(crate) fn encode<W: BufMut>(&self, out: &mut W) {
         out.write(Type::NEW_CONNECTION_ID);
         out.write_var(self.sequence);
         out.write_var(self.retire_prior_to);
@@ -839,7 +839,7 @@ impl NewConnectionId {
 }
 
 /// Smallest number of bytes this type of frame is guaranteed to fit within.
-pub const RETIRE_CONNECTION_ID_SIZE_BOUND: usize = 9;
+pub(crate) const RETIRE_CONNECTION_ID_SIZE_BOUND: usize = 9;
 
 /// An unreliable datagram
 #[derive(Debug, Clone)]
@@ -853,13 +853,13 @@ impl FrameStruct for Datagram {
 }
 
 impl Datagram {
-    pub(crate) fn encode<W: BufMut>(&self, length: bool, out: &mut W) {
+    pub(crate) fn encode(&self, length: bool, out: &mut BytesMut) {
         out.write(Type(*DATAGRAM_TYS.start() | u64::from(length))); // 1 byte
         if length {
             // Safe to unwrap because we check length sanity before queueing datagrams
             out.write(VarInt::from_u64(self.data.len() as u64).unwrap()); // <= 8 bytes
         }
-        out.put_slice(&self.data);
+        out.extend_from_slice(&self.data);
     }
 
     pub(crate) fn size(&self, length: bool) -> usize {
@@ -899,7 +899,7 @@ mod test {
                 assert_eq!(&packets[..], PACKETS);
                 assert_eq!(ack.ecn, Some(ECN));
             }
-            ref x => panic!("incorrect frame {:?}", x),
+            ref x => panic!("incorrect frame {x:?}"),
         }
     }
 }
